Method: Rack::Cache::CacheControl#reverse_max_age

Defined in:
lib/rack/cache/cache_control.rb

#reverse_max_ageObject Also known as: r_maxage

If a response includes a r-maxage directive, then for a reverse cache (but not for a private or proxy cache), the maximum age specified by this directive overrides the maximum age specified by either the max-age directive, the s-maxage directive, or the Expires header. The r-maxage directive also implies the semantics of the proxy-revalidate directive. i.e., that the reverse cache must not use the entry after it becomes stale to respond to a subsequent request without first revalidating it with the origin server. The r-maxage directive is always ignored by private and proxy caches.



137
138
139
# File 'lib/rack/cache/cache_control.rb', line 137

def reverse_max_age
  self['r-maxage'].to_i  if key?('r-maxage')
end